/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_BUBBLE_H__
#define __KIBA_BUBBLE_H__

#include <glib.h>
#include <cairo.h>

#include <kiba-types.h>
#include <kiba-cairo-utils.h>

G_BEGIN_DECLS

typedef struct _KibaBubble         KibaBubble;
typedef struct _KibaBubbleObject   KibaBubbleObject;
typedef struct _KibaBubbleClass    KibaBubbleClass;

#define KIBA_TYPE_BUBBLE           (kiba_bubble_get_type ())
#define KIBA_BUBBLE(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_BUBBLE, KibaBubble))
#define KIBA_BUBBLE_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_BUBBLE,  KibaBubbleClass))
#define KIBA_IS_BUBBLE(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_BUBBLE))
#define KIBA_IS_BUBBLE_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_BUBBLE))
#define KIBA_BUBBLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_BUBBLE, KibaBubbleClass))

enum {
	KIBA_BUBBLE_DRAWABLE = 0
};

typedef struct {
	union {
		KibaBackgroundStyle   style;
		char                 *file;
	} background;
	
	KibaCornerShape border_style;
	double background_alpha;
	double bg_affection;
	double bg_step_size;
	
	KibaRGBA bg_rgba_1;
	KibaRGBA bg_rgba_2;
	KibaRGBA bg_rgba_3;
	KibaRGBA border_rgba;
	KibaRGBA text_rgba;
}KibaBubbleStyle;

struct _KibaBubbleObject {
	GtkWidget        *widget;
	KibaBubble       *bubble;
	
	GdkRectangle      area;
	PangoLayout      *layout;
	char             *message;
	
	gint              hide_id;
	gint              timeout_id;
	double            sliding_offset;
	double            fade_factor;
	gboolean          fade_in;
};

struct _KibaBubble {
	GObject           parent;
	GtkWidget        *win;
	
	gint              flags;
	
	CairoSurfaceData  surface_data;
	
	KibaBubbleStyle   style;
	gint              border_width;
	double            slide_stepsize;
	char             *font;
};

struct _KibaBubbleClass {
	GObjectClass parent_class;
};

/**
 * Create a KibaBubble Instance.
 *
 * @return The KibaBubble Instance created.
 */
KibaBubble* kiba_bubble_new (GtkWidget *win);

/**
 * Draw all active KibaBubbleObject instances.
 *
 * @param bubble The KibaBubble instance.
 * @param cr The Drawing Context.
 */
void kiba_bubble_draw (KibaBubble *bubble,
		       cairo_t    *cr);

void kiba_bubble_set_style (KibaBubble       *bubble,
			    KibaBubbleStyle   style);

void kiba_bubble_set_border_width (KibaBubble *bubble,
			           gint        width);

void kiba_bubble_set_font (KibaBubble *bubble,
			   char       *font);

void kiba_bubble_set_flag (KibaBubble *bubble,
			   gint       flag);

void kiba_bubble_unset_flag (KibaBubble *bubble,
			     gint       flag);

void kiba_bubble_destroy (KibaBubble *bubble);

KibaBubbleObject* kiba_bubble_object_add (GtkWidget *widget);

gboolean kiba_bubble_object_is_shown (KibaBubbleObject *object);

/**
 * Set message for a KibaBubble Instance.
 *
 * @param bubble The KibaBubble Instance.
 * @param message The new Message.
 * @param hide_delay The Time (in ms) the message should be displayed, or 0.
 */
void kiba_bubble_object_set_message (KibaBubbleObject *object,
				     const char       *message,
				     int               hide_delay);

/**
 * UnSet message for a KibaBubble Instance.
 *
 * @param bubble The KibaBubble Instance.
 */
void kiba_bubble_object_unset_message (KibaBubbleObject *object);

void kiba_bubble_object_remove (KibaBubbleObject *object);

G_END_DECLS


#endif
