/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_DOCK_H__
#define __KIBA_DOCK_H__

#include <glib.h>
#include <gtk/gtk.h>

#include <kiba-object.h>
#include <kiba-cairo-utils.h>
#include <kiba-types.h>
#include <kiba-macros.h>

G_BEGIN_DECLS

#define KIBA_TYPE_DOCK           (kiba_dock_get_type ())
#define KIBA_DOCK(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_DOCK, KibaDock))
#define KIBA_DOCK_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_DOCK,  KibaDockClass))
#define KIBA_IS_DOCK(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_IS_DOCK_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_DOCK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_DOCK, KibaDockClass))

#define KIBA_DOCK_IS_HIDDEN(dock)    (KIBA_DOCK(dock)->autohide.state == DOCK_HIDDEN)
#define KIBA_DOCK_IS_UNHIDDEN(dock)  (KIBA_DOCK(dock)->autohide.state == DOCK_UNHIDDEN)
#define KIBA_DOCK_IS_DRAWABLE(dock)  GET_FLAG(KIBA_DOCK(dock)->flags, KIBA_DOCK_DRAWABLE)

#define DEFAULT_ICON_SIZE 40

typedef struct _KibaDock        KibaDock;
typedef struct _KibaDockClass   KibaDockClass;
typedef struct _KibaDockLayout  KibaDockLayout;

/**
 * Callback function to sort all Objects from the same group
 *
 * @param KibaDock The KibaDock Instance, associated to the group.
 * @param char The Group.
 * @param GList A unsorted List of all Objects from group.
 * @param gpointer User Data.
 * @return the sorted GList
 */
typedef GList* (*KibaGroupSortFunc) (KibaDock*, char*, const GList*, gpointer);

enum {
	KIBA_DOCK_AUTOHIDE = 0,
	KIBA_DOCK_DRAW_SEPARATOR,
	KIBA_DOCK_ROTATE_ADDED_OBJECT,
	KIBA_DOCK_SET_STRUT,
	KIBA_DOCK_PODEST_SHAPE,
	KIBA_DOCK_DRAWABLE
};

struct _KibaDockLayout {
	KibaPosition  position;
	gint          cap_size;
	gint          object_space;
	gint          object_margin;
	gint          object_size;
	gint          height;
	gint          max_size;
	gint          offset;
};

typedef struct {
	KibaCornerShape topleft;
	KibaCornerShape topright;
	KibaCornerShape bottomright;
	KibaCornerShape bottomleft;
	double          corner_radius;
}KibaDockShape;


typedef enum {
	DOCK_UNHIDDEN = 0,
	DOCK_UNHIDING,
	DOCK_HIDDEN,
	DOCK_HIDING
}KibaDockHideState;

typedef struct {
	KibaDockHideState state;
	gint              area;
	gint              speed;
	gint              delay;
}KibaAutoHide;

typedef struct {
	gboolean      draw;
	KibaDockShape shape;
	
	union {
		KibaBackgroundStyle   style;
		char                 *file;
	} background;
	
	double background_alpha;
	double bg_affection;
	double bg_step_size;
	double bg_radius_1;
	double bg_radius_2;
	
	double reflection_alpha;
	double reflection_scale_factor;
	
	double podest_height;
	
	KibaRGBA bg_rgba_1;
	KibaRGBA bg_rgba_2;
	KibaRGBA bg_rgba_3;
	KibaRGBA border_rgba;
	KibaRGBA prelight_rgba;
}KibaDockStyle;

struct _KibaDock {
	GtkBox          parent_instance;
	
	gint            flags;
	
	KibaDockLayout   d;
	double           scroll_offset;
	KibaDockStyle    style;
	KibaAutoHide     autohide;
	CairoSurfaceData surface_data;
	
	gint            bounce_height;
	gint            exec_bounce_height;
	double          rotate_speed;
	double          exec_rotate_speed;
	double          pulse_factor;
	double          exec_pulse_factor;
	
	GList          *groups;
};

struct _KibaDockClass {
	GtkBoxClass     parent_class;
	
	void (* position_changed) (KibaDock     *dock,
				   KibaPosition  position);
	void (* orientation_changed) (KibaDock       *dock,
				      GtkOrientation  orientation);
	void (* autohide_toggled) (KibaDock *dock,
				   gboolean  enabled);
	void (* object_size_changed) (KibaDock *dock,
				      gint      new_size);
};

/**
 * Creates and returns a new KibaDock Container.
 *
 * @return The KibaDock Container created.
 */
GtkWidget* kiba_dock_new (void);

void kiba_dock_set_group_sort_func (KibaDock         *dock,
				    const char       *group,
				    KibaGroupSortFunc func,
				    gpointer          data);

void kiba_dock_remove_group_sort_func (KibaDock    *dock,
				       const char  *group);

gboolean kiba_dock_object_is_on_dock (KibaDock   *dock,
				      KibaObject *object);

gboolean kiba_dock_akamaru_enabled (KibaDock *dock);

void kiba_dock_set_position (KibaDock     *dock,
			     KibaPosition  position);

KibaPosition kiba_dock_get_position (KibaDock *dock);

void kiba_dock_set_shape (KibaDock      *dock,
			  KibaDockShape  shape);

void kiba_dock_set_max_size (KibaDock *dock,
			     gint      max_size);

void kiba_dock_set_layout (KibaDock       *dock,
			   KibaDockLayout  d);

void kiba_dock_set_style (KibaDock       *dock,
			  KibaDockStyle   style);

void kiba_dock_set_flag (KibaDock *dock,
			 gint      flag);

void kiba_dock_unset_flag (KibaDock *dock,
			   gint      flag);

void kiba_dock_set_alpha (KibaDock *dock,
			  double    alpha);

void kiba_dock_set_object_size (KibaDock *dock,
				gint      size);

void kiba_dock_set_group_list (KibaDock *dock,
			       GSList   *group_list);

GList* kiba_dock_get_group_list (KibaDock *dock);

KibaDock* kiba_dock_get_for_id (const char *id);

/* FIXME: gtk_widget_size_request does not work in some cases (changing max_size) */
void kiba_dock_size_request (GtkWidget      *widget,
			     GtkRequisition *requisition);

void kiba_dock_expose (KibaDock *dock,
		       cairo_t  *cr);

G_END_DECLS

#endif
