/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_OBJECT_H__
#define __KIBA_OBJECT_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>

#include <kiba-title.h>
#include <kiba-bubble.h>
#include <kiba-progress.h>

G_BEGIN_DECLS

typedef struct _KibaObject         KibaObject;
typedef struct _KibaObjectClass    KibaObjectClass;

typedef void (*KibaObjectPaintFunc) (KibaObject*, cairo_t*, gpointer);

#define KIBA_TYPE_OBJECT           (kiba_object_get_type ())
#define KIBA_OBJECT(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_OBJECT, KibaObject))
#define KIBA_OBJECT_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TYPE_OBJECT,  KibaObjectClass))
#define KIBA_IS_OBJECT(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_IS_OBJECT_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_OBJECT, KibaObjectClass))

#define KIBA_OBJECT_IS_VISIBLE(obj)      GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_VISIBLE)
#define KIBA_OBJECT_IS_MOVEABLE(obj)     GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_MOVEABLE)
#define KIBA_OBJECT_IS_PAINTABLE(obj)    GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_PAINTABLE)
#define KIBA_OBJECT_IS_PRELIGHTABLE(obj) GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_PRELIGHTABLE)
#define KIBA_OBJECT_IS_ZOOMABLE(obj)     GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_ZOOMABLE)
#define KIBA_OBJECT_IS_BOUNCABLE(obj)    GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_BOUNCEABLE)
#define KIBA_OBJECT_IS_PULSEABLE(obj)    GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_PULSEABLE)
#define KIBA_OBJECT_IS_ROTATEABLE(obj)   GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_ROTATEABLE)
#define KIBA_OBJECT_IS_REFLECTABLE(obj)  GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_REFLECTABLE)
#define KIBA_OBJECT_IS_SWITCHABLE(obj)   GET_FLAG(KIBA_OBJECT(obj)->props, KIBA_OBJECT_SWITCHABLE)

enum {
	KIBA_OBJECT_VISIBLE = 1,
	KIBA_OBJECT_MOVEABLE,
	KIBA_OBJECT_PAINTABLE,
	KIBA_OBJECT_PRELIGHTABLE,
	KIBA_OBJECT_ZOOMABLE,
	KIBA_OBJECT_BOUNCEABLE,
	KIBA_OBJECT_PULSEABLE,
	KIBA_OBJECT_ROTATEABLE,
	KIBA_OBJECT_REFLECTABLE,
	KIBA_OBJECT_HAS_SHADOW,
	KIBA_OBJECT_SWITCHABLE,
};

typedef enum {
	KIBA_OBJECT_DRAGGING = 0,
	KIBA_OBJECT_PRELIGHTED,
	KIBA_OBJECT_ROTATING,
	KIBA_OBJECT_PULSING,
	KIBA_OBJECT_BOUNCING,
	KIBA_OBJECT_URGENT,
	KIBA_OBJECT_ACTIVE,
	KIBA_OBJECT_MOVING,
	KIBA_OBJECT_FADE_IN,
	KIBA_OBJECT_FADE_OUT,
	KIBA_OBJECT_SWITCHING
}KibaObjectState;

typedef enum {
	KIBA_OBJECT_WINDOW_VISIBLE = 0,
	KIBA_OBJECT_WINDOW_INVISIBLE
}KibaObjectWindowState;

typedef enum {
	KIBA_OBJECT_ON_DOCK = 0,
	KIBA_OBJECT_LEFT_OF_DOCK,
	KIBA_OBJECT_RIGHT_OF_DOCK
}KibaObjectPosition;

struct _KibaObject {
	GtkBin              parent;
	
	KibaObjectPosition  position;
	
	KibaObjectWindowState win_state;
	KibaObjectState       state;
	gint                  props;
	gboolean              opaque;
	
	double              bounce_offset;
	double              alpha;          /* current alpha value */
	
	double              destroy_factor;
	
	double              scroll_alpha;
	
	gint               *size;           /* pointer to dock->d.object_zize,
					       or to dock->d.height for fixed objects */
	double              scale_x;        /* width scale factor */
	double              scale_y;        /* height scale factor */
	
	GtkAllocation       allocation;
	gchar              *group;    /* used to stick objects from the same group together,
					 for plugins this is the plugin id (example: launcher) */
	
	KibaTitleObject    *title;
	KibaBubbleObject   *bubble;
	KibaProgressObject *progress;
	
	CairoSurfaceData   *surface_data;
	cairo_surface_t    *switch_surface; /* used for the switch animation */
	
	KibaObjectPaintFunc paint;
	gpointer            paint_data;
	
	GdkWindow          *akamaru_window; /* toplevel window for akamaru, or NULL if akamaru is disabled */
};

struct _KibaObjectClass {
	GtkBinClass parent_class;
	
	void (* group_set) (KibaObject  *dock,
			    const gchar *group);
	void (* urgent_toggled) (KibaObject  *dock,
				 gboolean     urgent);
	void (* active_toggled) (KibaObject  *dock,
				 gboolean     active);
};

/**
 * Creates and returns a KibaObject Container.
 */
GtkWidget* kiba_object_new (KibaObjectWindowState state,
			    gboolean              opaque);

GtkWidget* kiba_object_new_for_widget (GtkWidget *widget);

void kiba_object_set_urgent (KibaObject *object,
			     gboolean    urgent);

gboolean kiba_object_get_urgent (KibaObject *object);

void kiba_object_set_reflectable (KibaObject *object,
				  gboolean    reflect);

gboolean kiba_object_get_reflectable (KibaObject *object);

void kiba_object_set_switchable (KibaObject *object,
				 gboolean    switchable);

void kiba_object_set_has_shadow (KibaObject *object,
				 gboolean    has_shadow);

gboolean kiba_object_get_has_shadow (KibaObject *object);

void kiba_object_set_title (KibaObject *object,
                            const char *title);

const char* kiba_object_get_title (KibaObject *object);

void kiba_object_show_title (GtkWidget *widget);

void kiba_object_hide_title (GtkWidget *widget);

void kiba_object_set_active (GtkWidget *widget,
			     gboolean   active);

gboolean kiba_object_get_active (GtkWidget *widget);

void kiba_object_set_prelight (GtkWidget *widget);

void kiba_object_unset_prelight (GtkWidget *widget);

void kiba_object_start_pulsing (GtkWidget *widget,
				double     factor);

void kiba_object_stop_pulsing (GtkWidget *widget);

void kiba_object_start_rotation (GtkWidget *widget,
				 double     speed);

void kiba_object_stop_rotation (GtkWidget *widget);

void kiba_object_start_bouncing (GtkWidget *widget,
				 double     bounce_height);

void kiba_object_stop_bouncing (GtkWidget *widget);

void kiba_object_set_alpha (KibaObject *object,
			    double      alpha);

void kiba_object_set_group (KibaObject  *object,
			    const gchar *group);

void kiba_object_set_icon_geometry (KibaObject *object,
				    GdkWindow  *window);

void kiba_object_set_requisition_factor (KibaObject *object,
					 double      scale_x,
					 double      scale_y);

void kiba_object_size_request (GtkWidget      *widget,
			       GtkRequisition *requisition);

void kiba_object_resize (KibaObject *object);

/**
Rendering
**/

void kiba_object_queue_draw (KibaObject *object);

void kiba_object_set_paint_function (KibaObject          *object,
				     KibaObjectPaintFunc  func,
				     gpointer             data);

void kiba_object_paint (KibaObject *object,
		        cairo_t    *cr,
		        gpointer    data);

void kiba_object_paint_reflection (KibaObject *object,
				   cairo_t    *cr);

void kiba_object_paint_shadow (KibaObject *object,
			       cairo_t    *cr);

void kiba_object_paint_with_prelight (KibaObject       *object,
				      cairo_surface_t  *surface,
				      cairo_t          *cr);

void kiba_object_rotate (KibaObject *object,
			 cairo_t    *cr);

void kiba_object_paint_pulse (KibaObject *object,
			      cairo_t    *cr);

void kiba_object_switch_surface (KibaObject *object);

double kiba_object_get_switch_factor (KibaObject *object);


void kiba_object_destroy (KibaObject *object);

double kiba_object_get_destroy_factor (KibaObject *object);

gboolean kiba_object_get_destroying (KibaObject *object);

G_END_DECLS

#endif
